--[[
Name: MythicPlayerInfo

Copyright 2018 Amegez,MerguezChipo

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
]]

ACTIVITY_IDS={};
ACTIVITY_IDS[197]="EoA";
ACTIVITY_IDS[198]="DT";
ACTIVITY_IDS[199]="BRH";
ACTIVITY_IDS[200]="HoV";
ACTIVITY_IDS[206]="NL";
ACTIVITY_IDS[207]="VoW";
ACTIVITY_IDS[208]="MoS";
ACTIVITY_IDS[209]="Arc";
ACTIVITY_IDS[210]="CoS";
ACTIVITY_IDS[227]="RKL";
ACTIVITY_IDS[233]="CoEN";
ACTIVITY_IDS[234]="RKH";
ACTIVITY_IDS[239]="SoT";
ACTIVITY_IDS[244]="AD";
ACTIVITY_IDS[245]="FH";
ACTIVITY_IDS[246]="TD";
ACTIVITY_IDS[247]="ML";
ACTIVITY_IDS[248]="WM";
ACTIVITY_IDS[249]="KR";
ACTIVITY_IDS[250]="TOS";
ACTIVITY_IDS[251]="UNDR";
ACTIVITY_IDS[252]="SOTS";
ACTIVITY_IDS[353]="SIEGE";

MythicPlayerInfo = {};

C_ChatInfo.RegisterAddonMessagePrefix("MythicPlayerInfo");
local defaultGTTFontName = "";
local defaultGTTFontSize = 0;

local f = CreateFrame("Frame")
f:SetScript("OnEvent", function(e,EventName,Data1,Data2)
	if(EventName == "CHAT_MSG_ADDON_LOGGED" and Data1 == "MythicPlayerInfo")then
		if string.find(Data2, "01", 1) == 1 then -- Collect player mythic info
			MPI_setPlayerInfo(string.sub(Data2,4));
			MPI_sharePlayerInfo(string.sub(Data2,4));
		elseif string.find(Data2, "02", 1) == 1 then -- Request player mythic info
			MPI_sendPlayerInfo(string.sub(Data2,4));
		elseif  string.find(Data2, "03", 1) == 1 then -- Share player mythic info to party
			MPI_setPlayerInfo(string.sub(Data2,4));
		end
	elseif (EventName == "LFG_LIST_APPLICATION_STATUS_UPDATED" and Data2 == "applied")then
		local _, _, _, _, _, _, _, _, _, _, _, _, leader = C_LFGList.GetSearchResultInfo(Data1);
		C_ChatInfo.SendAddonMessageLogged("MythicPlayerInfo", MPI_GetPlayerInfo(), "WHISPER", leader);
	elseif (EventName == "PLAYER_ENTERING_WORLD")then
		C_MythicPlus.RequestCurrentAffixes();
		C_MythicPlus.RequestMapInfo();
		C_MythicPlus.RequestRewards();
	elseif (EventName == "PARTY_MEMBERS_CHANGED")then
		
	end
end)
f:RegisterEvent("CHAT_MSG_ADDON_LOGGED");
f:RegisterEvent("LFG_LIST_APPLICATION_STATUS_UPDATED");
f:RegisterEvent("PLAYER_ENTERING_WORLD");
--f:RegisterEvent("PARTY_MEMBERS_CHANGED");

function MPI_ShowToolTip(self)
	local name, class, localizedClass, level, itemLevel = C_LFGList.GetApplicantMemberInfo(self:GetParent().applicantID,1);
	MPI_SetTooltipPlayerInfo(name, class, localizedClass, level, itemLevel);
end

function MPI_HideToolTip(self)
	MPI_GameTooltip:Hide();
	MPI_GameTooltip_ClearLines();
end

function MPI_setPlayerInfo(data)
	local info = {strsplit("#",data)};
	MythicPlayerInfo[info[1]] = {};
	for i=2,#info,4 do
		table.insert(MythicPlayerInfo[info[1]],{info[i],info[i+1],info[i+2],info[i+3]});
	end
end

function MPI_GetPlayerInfo()
	--local playerInfo = string.format("%s-%s",UnitFullName("player"));
	local playerName = UnitFullName("player");
	local playerInfo = "01#"..playerName;
	for k,v in pairs(ACTIVITY_IDS) do
		local seasonBestDurationSec, seasonBestLevel, _, _, members = C_MythicPlus.GetSeasonBestForMap(k);
		if seasonBestLevel then
			local role = 0;
			for i, member in ipairs(members) do
				if (member.name and member.name == playerName) then
					role = member.specID;
				end
			end
			playerInfo = string.format("%s#%s#%s#%s#%s",playerInfo,k,seasonBestDurationSec,seasonBestLevel,role);
		end
	end
	return playerInfo;
end

function MPI_sendPlayerInfo(requestFrom)
	C_ChatInfo.SendAddonMessageLogged("MythicPlayerInfo", MPI_GetPlayerInfo(), "WHISPER", requestFrom);
end

function MPI_sharePlayerInfo(data)
	--[[
	for i = 1,MAX_PARTY_MEMBERS do
		local name = string.format("%s-%s",UnitFullName("party"..i));
		if name ~= string.format("%s-%s",UnitFullName("player")) then
			
		end
	end
	]]--
end

function MPI_requestPlayerInfo(requestToPlayer)
	local playerName = string.format("%s-%s",UnitFullName("player"));
	C_ChatInfo.SendAddonMessageLogged("MythicPlayerInfo", "02#"..playerName, "WHISPER", requestToPlayer);
end

function MPI_SetTooltipPlayerInfo(name, class, localizedClass, level, itemLevel)
	MPI_GameTooltip_ClearLines();
	
	if class then
		local classTextColor = grayedOut and GRAY_FONT_COLOR or RAID_CLASS_COLORS[class];
		MPI_GameTooltip_AddLine(name,RAID_CLASS_COLORS[class].colorStr); --,classTextColor.r, classTextColor.g, classTextColor.b);
		MPI_GameTooltip_AddLine(string.format(UNIT_TYPE_LEVEL_TEMPLATE, level, localizedClass));
		MPI_GameTooltip_AddLine(string.format(LFG_LIST_ITEM_LEVEL_CURRENT, itemLevel));
	end
	
	local score = 0;
	local str = {};
	MPI_GameTooltip_AddLine("MythicPlayerInfo",RAID_CLASS_COLORS.ROGUE.colorStr);
	if MythicPlayerInfo[name] then
		for k,v in pairs(MythicPlayerInfo[name]) do
			local mname,_,timer = C_ChallengeMode.GetMapUIInfo(v[1]);
			mname = MPI_formatString(mname,30);
			score = score + ((v[3]*10)+((timer-v[2])/(timer/10)));
			local oyu = 0;
			if timer-v[2] >= 0 then
				oyu = string.format("|cffabd473Under by %s|r",GetTimeStringFromSeconds(timer-v[2]));
			else
				oyu = string.format("|cffc41f3bOver  by %s|r",GetTimeStringFromSeconds(v[2]-timer));
			end
			local dtime = GetTimeStringFromSeconds(v[2]);
			local _, name = GetSpecializationInfoByID(tonumber(v[4]));
			name = MPI_formatString(name,15);
			local role = GetSpecializationRoleByID(tonumber(v[4]));
			local texture = CreateAtlasMarkup("roleicon-tiny-dps");
			if (role == "TANK") then
				texture = CreateAtlasMarkup("roleicon-tiny-tank");
			elseif (role == "DAMAGER") then
				texture = CreateAtlasMarkup("roleicon-tiny-dps");
			elseif (role == "HEALER") then
				texture = CreateAtlasMarkup("roleicon-tiny-healer");
			end
			local lvel = MPI_formatString(v[3],2);
			str[k] = string.format("%s %s +%s %s (%s)",mname,MYTHIC_PLUS_LEADER_BOARD_NAME_ICON:format(texture,name),lvel,dtime,oyu);
		end
		MPI_GameTooltip_AddLine(string.format("Score : %.2f",score),RAID_CLASS_COLORS.MONK.colorStr);
		for k,v in pairs(str) do
			MPI_GameTooltip_AddLine(v);		
		end
	else
		MPI_GameTooltip_AddLine("Score : "..score,RAID_CLASS_COLORS.MONK.colorStr);
	end
	MPI_GameTooltip:Show();
end

function MPI_formatString(str, maxChar)
	local specialChars = strtrim(str,'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890 -_\'!');
	local toRemove = floor(#str-(#specialChars/2));
	return str..string.rep(' ', maxChar-toRemove);
end

function MPI_ButtonShow(self)
	for i=1,#self.Members do
		local b = self.Members[i];
		b:HookScript("OnEnter", MPI_ShowToolTip);
		b:HookScript("OnLeave", MPI_HideToolTip);
	end
end

function MPI_GameTooltip_AddLine(text,color)
	if color then
		text = "|c"..color..text.."|r";
	else
		text = "|c"..RAID_CLASS_COLORS.PRIEST.colorStr..text.."|r";
	end
	if(MPI_GameTooltip.text:GetText())then
		MPI_GameTooltip.text:SetText(MPI_GameTooltip.text:GetText().."\r\n"..text);
	else
		MPI_GameTooltip.text:SetText(text);
	end
end

function MPI_GameTooltip_ClearLines()
	MPI_GameTooltip.text:SetText("");
end

hooksecurefunc("LFGListUtil_SetSearchEntryTooltip", function(tooltip, resultID, autoAcceptOption)
	local id, _, _, _, _, _, _, _, _, _, _, _, leaderName ,numMembers = C_LFGList.GetSearchResultInfo(resultID);
	local leaderShort = {strsplit("-",leaderName)};
	print(leaderShort[1]);
	MPI_GameTooltip_ClearLines();
	if not MythicPlayerInfo[leaderShort[1]] then
		print("send");
		MPI_requestPlayerInfo(leaderName);
	else
		MPI_SetTooltipPlayerInfo(leaderShort[1]);
	end
end)

for i=1,14 do
	local b = _G["LFGListApplicationViewerScrollFrameButton"..i];
	b:HookScript("OnShow", MPI_ButtonShow);
end

if not MPI_GameTooltip then
   MPI_GameTooltip = CreateFrame("Frame",nil,GroupFinderFrame);
   MPI_GameTooltip:SetPoint("TOPLEFT",GroupFinderFrame:GetWidth(),0);
   MPI_GameTooltip:SetSize(700,GroupFinderFrame:GetHeight());
   MPI_GameTooltip:SetBackdrop({
         bgFile = "Interface/DialogFrame/UI-DialogBox-Background",
         edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
         tile = true, tileSize = 32, edgeSize = 16,
         insets = { left = 1, right = 1, top = 1, bottom = 1 }});
   MPI_GameTooltip.text = MPI_GameTooltip:CreateFontString("MPI_GameTooltip","ARTWORK");
end
MPI_GameTooltip.text:SetPoint("TOPLEFT", 10, -10);
MPI_GameTooltip.text:SetTextColor(0.85, 0.85, 0.85, 1);
MPI_GameTooltip.text:SetFont("Interface\\AddOns\\MythicPlayerInfo\\fonts\\VeraMono.ttf", 12);
MPI_GameTooltip.text:SetJustifyH("LEFT");
MPI_GameTooltip.text:SetText("");



